--- /dev/null	2012-06-13 12:04:41.000000000 +0000
+++ new/usr/src/uts/common/inet/dccp/dccp_stats.c	2012-06-13 12:04:40.933848015 +0000
@@ -0,0 +1,153 @@
+/*
+ * CDDL HEADER START
+ *
+ * The contents of this file are subject to the terms of the
+ * Common Development and Distribution License (the "License").
+ * You may not use this file except in compliance with the License.
+ *
+ * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
+ * or http://www.opensolaris.org/os/licensing.
+ * See the License for the specific language governing permissions
+ * and limitations under the License.
+ *
+ * When distributing Covered Code, include this CDDL HEADER in each
+ * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
+ * If applicable, add the following below this CDDL HEADER, with the
+ * fields enclosed by brackets "[]" replaced with your own identifying
+ * information: Portions Copyright [yyyy] [name of copyright owner]
+ *
+ * CDDL HEADER END
+ */
+
+/*
+ * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
+ * Use is subject to license terms.
+ */
+
+/*
+ * Copyright 2012 David Hoeppner.  All rights reserved.
+ */
+
+/*
+ * Functions related to MIB-II and kstat.
+ */
+
+#include <sys/types.h>
+#include <sys/tihdr.h>
+#include <sys/policy.h>
+#include <sys/tsol/tnet.h>
+
+#include <inet/common.h>
+#include <inet/ip.h>
+#include <inet/kstatcom.h>
+#include <inet/snmpcom.h>
+
+#include <sys/cmn_err.h>
+
+#include "dccp_impl.h"
+
+static int dccp_snmp_state(dccp_t *);
+
+static int
+dccp_snmp_state(dccp_t *dccp)
+{
+	if (dccp == NULL) {
+		return (0);
+	}
+
+	switch(dccp->dccp_state) {
+	case DCCPS_CLOSED:
+		return (MIB2_DCCP_closed);
+	default:
+		return (0);
+	}
+}
+
+/*
+ * Get the MIB-II stats.
+ */
+mblk_t *
+dccp_snmp_get(queue_t *q, mblk_t *mpctl)
+{
+	conn_t		*connp = Q_TO_CONN(q);
+	connf_t		*connfp;
+	ip_stack_t	*ips;
+	dccp_stack_t	*dccps;
+	struct opthdr	*optp;
+	mblk_t		*mp2ctl;
+	mblk_t		*mpdata;
+	mblk_t		*mp_conn_ctl = NULL;
+	mblk_t		*mp_conn_tail;
+	mblk_t		*mp_attr_ctl = NULL;
+	mblk_t		*mp_attr_tail;
+	size_t		dccp_mib_size;
+	size_t		dce_size;
+	zoneid_t	zoneid;
+	int		i;
+	mib2_dccpConnEntry_t	dce;
+
+	mp2ctl = copymsg(mpctl);
+
+	if (mpctl == NULL ||
+	    (mpdata = mpctl->b_cont) == NULL ||
+	    (mp_conn_ctl = copymsg(mpctl)) == NULL ||
+	    (mp_attr_ctl = copymsg(mpctl)) == NULL) {
+		freemsg(mp_conn_ctl);
+		freemsg(mp_attr_ctl);
+		freemsg(mpctl);
+		freemsg(mp2ctl);
+		return (NULL);
+	}
+
+	ips = connp->conn_netstack->netstack_ip;
+	dccps = connp->conn_netstack->netstack_dccp;
+	dce_size = sizeof (mib2_dccpConnEntry_t);
+	zoneid = Q_TO_CONN(q)->conn_zoneid;
+
+	for (i = 0; i < CONN_G_HASH_SIZE; i++) {
+		ips = dccps->dccps_netstack->netstack_ip;
+
+		connfp = &ips->ips_ipcl_globalhash_fanout[i];
+		connp = NULL;
+
+		while ((connp = ipcl_get_next_conn(connfp, connp,
+		    IPCL_DCCPCONN)) != NULL) {
+			dccp_t	*dccp;
+
+			if (connp->conn_zoneid != zoneid) {
+				continue;
+			}
+
+			dccp = connp->conn_dccp;
+
+			dce.dccpConnState = dccp_snmp_state(dccp);
+
+			if (connp->conn_ipversion == IPV4_VERSION ||
+			    (dccp->dccp_state <= DCCPS_LISTEN)) {
+				dce.dccpConnRemAddress =
+				    connp->conn_faddr_v4;
+				dce.dccpConnLocalAddress =
+				    connp->conn_laddr_v4;
+			}
+
+			dce.dccpConnLocalPort = ntohs(connp->conn_lport);
+			dce.dccpConnRemPort = ntohs(connp->conn_fport);
+
+			dce.dccpConnCreationProcess = (connp->conn_cpid < 0) ?
+			    MIB2_UNKNOWN_PROCESS : connp->conn_cpid;
+			dce.dccpConnCreationTime = connp->conn_open_time;
+
+			(void) snmp_append_data2(mp_conn_ctl->b_cont,
+			    &mp_conn_tail, (char *)&dce, dce_size);
+		}
+	}
+
+	optp = (struct opthdr *)&mp_conn_ctl->b_rptr[
+	    sizeof (struct T_optmgmt_ack)];
+	optp->level = MIB2_DCCP;
+	optp->name = MIB2_DCCP_CONN;
+	optp->len = msgdsize(mp_conn_ctl->b_cont);
+	qreply(q, mp_conn_ctl);
+
+	return (mp2ctl);
+}
